/***********************************************************************
 *      This example program is an excerpt from the book:              *
 *                                                                     *
 *          X Window OSF/Motif Toolkit Programming (I)                 *
 *            - Programming Basics, Resource and Event -               *
 *                                                                     *
 *      Copyright 1994, Hideyuki Hayashi                               *
 *                                                                     *
 *      This program can be distributed without fee, provided          *
 *      that the above copyright notice appear in all copies.          *
 ***********************************************************************/

#include <Xm/XmAll.h>

/* Callback for "File" menu */
static void FileMenuCB(w, client_data, call_data)
    Widget    w;
    XtPointer client_data;
    XtPointer call_data;
{
    int menu_item = (int)client_data;  /* item number of selected menu */
    printf("File Menu: item %d is selected\n", menu_item);
    if (menu_item == 3)  /* If the selected item was 3 ("Exit"), */
        exit(0);         /* exit this program */
}

/* Copy the selected text to clipboard */
static void CopyCB(w, client_data, call_data)
    Widget    w;
    XtPointer client_data;
    XtPointer call_data;
{
    Widget text = (Widget)client_data;
    XmTextCopy(text, XtLastTimestampProcessed(XtDisplay(text)));
}

/* Cut the selected text to clipboard */
static void CutCB(w, client_data, call_data)
    Widget    w;
    XtPointer client_data;
    XtPointer call_data;
{
    Widget text = (Widget)client_data;
    XmTextCut(text, XtLastTimestampProcessed(XtDisplay(text)));
}

/* Paste the data in clipboard into text */
static void PasteCB(w, client_data, call_data)
    Widget    w;
    XtPointer client_data;
    XtPointer call_data;
{
    Widget text = (Widget)client_data;
    XmTextPaste(text);
}

/* Create menubar and pulldown menu */
static Widget CreateMenu(parent)
    Widget parent;
{
    Widget  menubar, filemenu;

    /* Create menu bar */
    menubar = XmVaCreateSimpleMenuBar(parent, "menubar",
                  XmVaCASCADEBUTTON, NULL, NULL,  /* "File" menu */
                  NULL);
    XtManageChild(menubar);

    /* Create "File" pulldown menu */
    filemenu = XmVaCreateSimplePulldownMenu(menubar, "filemenu", 0, FileMenuCB,
                  XmVaPUSHBUTTON, NULL, NULL, NULL, NULL,  /* "New" */
                  XmVaPUSHBUTTON, NULL, NULL, NULL, NULL,  /* "Open" */
                  XmVaPUSHBUTTON, NULL, NULL, NULL, NULL,  /* "Save" */
                  XmVaSEPARATOR,  /* Separator */
                  XmVaPUSHBUTTON, NULL, NULL, NULL, NULL,  /* "Exit" */
                  NULL);

    return(menubar);
}

main(argc, argv)
    int  argc;
    char **argv;
{
    XtAppContext app_context;
    Widget  toplevel, mainwin, menubar, work, text, command, copy, cut, paste;
    Arg     al[20];
    int     ac;

    XtSetLanguageProc(NULL, NULL, NULL);

    ac = 0;
    toplevel = XtAppInitialize(&app_context, "Menuaccel", NULL, 0,
                               &argc, argv, NULL, al, ac);

    ac = 0;
    XtSetArg(al[ac], XmNshowSeparator, True); ac++;
    XtSetArg(al[ac], XmNcommandWindowLocation, XmCOMMAND_BELOW_WORKSPACE); ac++;
    mainwin = XmCreateMainWindow(toplevel, "mainwin", al, ac);
    XtManageChild(mainwin);

    menubar = CreateMenu(mainwin); /* Create menu */

    ac = 0;
    work = XmCreateRowColumn(mainwin, "work", al, ac);
    XtManageChild(work);

    ac = 0;
    command = XmCreateRowColumn(mainwin, "command", al, ac);
    XtManageChild(command);

    XmMainWindowSetAreas(mainwin, menubar, command, NULL, NULL, work);

    ac = 0;
    text = XmCreateText(work, "text", al, ac);
    XtManageChild(text);

    ac = 0;
    copy = XmCreatePushButton(command, "copy", al, ac);
    XtManageChild(copy);
    ac = 0;
    cut = XmCreatePushButton(command, "cut", al, ac);
    XtManageChild(cut);
    ac = 0;
    paste = XmCreatePushButton(command, "paste", al, ac);
    XtManageChild(paste);

    XtAddCallback(copy,  XmNactivateCallback, CopyCB,  text);
    XtAddCallback(cut,   XmNactivateCallback, CutCB,   text);
    XtAddCallback(paste, XmNactivateCallback, PasteCB, text);

    /* Install accelerator on the text widget */
    XtInstallAllAccelerators(text, command);

    XtRealizeWidget(toplevel);
    XtAppMainLoop(app_context);
}
